@@ -376,7 +376,7 @@ extern int base_yydebug;
     REASSIGN = 577,                /* REASSIGN  */
     RECHECK = 578,                 /* RECHECK  */
     RECURSIVE = 579,               /* RECURSIVE  */
-    REF = 580,                     /* REF  */
+    REF_P = 580,                   /* REF_P  */
     REFERENCES = 581,              /* REFERENCES  */
     REFERENCING = 582,             /* REFERENCING  */
     REFRESH = 583,                 /* REFRESH  */
