@@ -26,9 +26,11 @@
 #include "catalog/indexing.h"
 #include "catalog/namespace.h"
 #include "catalog/objectaccess.h"
+#include "catalog/pg_inherits.h"
 #include "catalog/pg_rewrite.h"
 #include "catalog/storage.h"
 #include "commands/policy.h"
+#include "commands/tablecmds.h"
 #include "miscadmin.h"
 #include "nodes/nodeFuncs.h"
 #include "parser/parse_utilcmd.h"
@@ -413,13 +415,17 @@ DefineQueryRewrite(const char *rulename,
 		 * Are we converting a relation to a view?
 		 *
 		 * If so, check that the relation is empty because the storage for the
-		 * relation is going to be deleted.  Also insist that the rel not have
-		 * any triggers, indexes, child tables, policies, or RLS enabled.
-		 * (Note: these tests are too strict, because they will reject
-		 * relations that once had such but don't anymore.  But we don't
-		 * really care, because this whole business of converting relations to
-		 * views is just a kluge to allow dump/reload of views that
-		 * participate in circular dependencies.)
+		 * relation is going to be deleted.  Also insist that the rel not be
+		 * involved in partitioning, nor have any triggers, indexes, child or
+		 * parent tables, RLS policies, or RLS enabled.  (Note: some of these
+		 * tests are too strict, because they will reject relations that once
+		 * had such but don't anymore.  But we don't really care, because this
+		 * whole business of converting relations to views is just an obsolete
+		 * kluge to allow dump/reload of views that participate in circular
+		 * dependencies.)
+		 *
+		 * Also ensure the relation isn't being manipulated in any outer SQL
+		 * command of our own session.
 		 */
 		if (event_relation->rd_rel->relkind != RELKIND_VIEW &&
 			event_relation->rd_rel->relkind != RELKIND_MATVIEW)
@@ -428,12 +434,17 @@ DefineQueryRewrite(const char *rulename,
 			Snapshot	snapshot;
 			TupleTableSlot *slot;
 
+			CheckTableNotInUse(event_relation, "CREATE RULE");
+
 			if (event_relation->rd_rel->relkind == RELKIND_PARTITIONED_TABLE)
 				ereport(ERROR,
 						(errcode(ERRCODE_WRONG_OBJECT_TYPE),
 						 errmsg("cannot convert partitioned table \"%s\" to a view",
 								RelationGetRelationName(event_relation))));
 
+			/* only case left: */
+			Assert(event_relation->rd_rel->relkind == RELKIND_RELATION);
+
 			if (event_relation->rd_rel->relispartition)
 				ereport(ERROR,
 						(errcode(ERRCODE_WRONG_OBJECT_TYPE),
@@ -471,6 +482,12 @@ DefineQueryRewrite(const char *rulename,
 						 errmsg("could not convert table \"%s\" to a view because it has child tables",
 								RelationGetRelationName(event_relation))));
 
+			if (has_superclass(RelationGetRelid(event_relation)))
+				ereport(ERROR,
+						(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+						 errmsg("could not convert table \"%s\" to a view because it has parent tables",
+								RelationGetRelationName(event_relation))));
+
 			if (event_relation->rd_rel->relrowsecurity)
 				ereport(ERROR,
 						(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
@@ -521,6 +538,18 @@ DefineQueryRewrite(const char *rulename,
 								RelationGetDescr(event_relation),
 								false, false);
 		}
+
+		/*
+		 * And finally, if it's not an ON SELECT rule then it must *not* be
+		 * named _RETURN.  This prevents accidentally or maliciously replacing
+		 * a view's ON SELECT rule with some other kind of rule.
+		 */
+		if (strcmp(rulename, ViewSelectRuleName) == 0)
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+					 errmsg("non-view rule for \"%s\" must not be named \"%s\"",
+							RelationGetRelationName(event_relation),
+							ViewSelectRuleName)));
 	}
 
 	/*
