@@ -3,7 +3,7 @@
  * compat.c
  *		Reimplementations of various backend functions.
  *
- * Portions Copyright (c) 2013-2019, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2013-2020, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
  *		src/bin/pg_waldump/compat.c
@@ -21,7 +21,6 @@
 #include <time.h>
 
 #include "utils/datetime.h"
-#include "lib/stringinfo.h"
 
 /* copied from timestamp.c */
 pg_time_t
@@ -63,29 +62,3 @@ timestamptz_to_str(TimestampTz dt)
 
 	return buf;
 }
-
-/*
- * Provide a hacked up compat layer for StringInfos so xlog desc functions can
- * be linked/called.
- */
-void
-appendStringInfo(StringInfo str, const char *fmt,...)
-{
-	va_list		args;
-
-	va_start(args, fmt);
-	vprintf(fmt, args);
-	va_end(args);
-}
-
-void
-appendStringInfoString(StringInfo str, const char *string)
-{
-	appendStringInfo(str, "%s", string);
-}
-
-void
-appendStringInfoChar(StringInfo str, char ch)
-{
-	appendStringInfo(str, "%c", ch);
-}
