@@ -146,16 +146,35 @@ my @options = (
 	[
 		'invalid init step',
 		'-i -I dta',
-		[ qr{unrecognized initialization step}, qr{allowed steps are} ]
+		[
+			qr{unrecognized initialization step},
+			qr{Allowed step characters are}
+		]
 	],
 	[
 		'bad random seed',
 		'--random-seed=one',
 		[
-			qr{unrecognized random seed option "one": expecting an unsigned integer, "time" or "rand"},
+			qr{unrecognized random seed option "one"},
+			qr{Expecting an unsigned integer, "time" or "rand"},
 			qr{error while setting random seed from --random-seed option}
 		]
 	],
+	[
+		'bad partition method',
+		'-i --partition-method=BAD',
+		[ qr{"range"}, qr{"hash"}, qr{"BAD"} ]
+	],
+	[
+		'bad partition number',
+		'-i --partitions -1',
+		[qr{invalid number of partitions: "-1"}]
+	],
+	[
+		'partition method without partitioning',
+		'-i --partition-method=hash',
+		[qr{partition-method requires greater than zero --partitions}]
+	],
 
 	# logging sub-options
 	[
@@ -217,6 +236,17 @@ pgbench(
 	],
 	'pgbench builtin list');
 
+# builtin listing
+pgbench(
+	'--show-script se',
+	0,
+	[qr{^$}],
+	[
+		qr{select-only: }, qr{SELECT abalance FROM pgbench_accounts WHERE},
+		qr{(?!UPDATE)},    qr{(?!INSERT)}
+	],
+	'pgbench builtin listing');
+
 my @script_tests = (
 
 	# name, err, { file => contents }
@@ -305,6 +335,11 @@ my @script_tests = (
 		'double overflow 3',
 		[qr{double constant overflow}],
 		{ 'overflow-3.sql' => "\\set d .1E310\n" }
+	],
+	[
+		'set i',
+		[ qr{set i 1 }, qr{\^ error found here} ],
+		{ 'set_i_op' => "\\set i 1 +\n" }
 	],);
 
 for my $t (@script_tests)
