@@ -1,15 +1,16 @@
 # Test for timeline switch
-# Ensure that a cascading standby is able to follow a newly-promoted standby
-# on a new timeline.
 use strict;
 use warnings;
 use File::Path qw(rmtree);
 use PostgresNode;
 use TestLib;
-use Test::More tests => 1;
+use Test::More tests => 3;
 
 $ENV{PGDATABASE} = 'postgres';
 
+# Ensure that a cascading standby is able to follow a newly-promoted standby
+# on a new timeline.
+
 # Initialize master node
 my $node_master = get_new_node('master');
 $node_master->init(allows_streaming => 1);
@@ -37,18 +38,22 @@ $node_master->safe_psql('postgres',
 $node_master->wait_for_catchup($node_standby_1, 'replay',
 	$node_master->lsn('write'));
 
-# Stop and remove master, and promote standby 1, switching it to a new timeline
+# Stop and remove master
 $node_master->teardown_node;
-$node_standby_1->promote;
+
+# promote standby 1 using "pg_promote", switching it to a new timeline
+my $psql_out = '';
+$node_standby_1->psql(
+	'postgres',
+	"SELECT pg_promote(wait_seconds => 300)",
+	stdout => \$psql_out);
+is($psql_out, 't', "promotion of standby with pg_promote");
 
 # Switch standby 2 to replay from standby 1
-rmtree($node_standby_2->data_dir . '/recovery.conf');
 my $connstr_1 = $node_standby_1->connstr;
 $node_standby_2->append_conf(
-	'recovery.conf', qq(
-primary_conninfo='$connstr_1 application_name=@{[$node_standby_2->name]}'
-standby_mode=on
-recovery_target_timeline='latest'
+	'postgresql.conf', qq(
+primary_conninfo='$connstr_1'
 ));
 $node_standby_2->restart;
 
@@ -62,3 +67,42 @@ $node_standby_1->wait_for_catchup($node_standby_2, 'replay',
 my $result =
   $node_standby_2->safe_psql('postgres', "SELECT count(*) FROM tab_int");
 is($result, qq(2000), 'check content of standby 2');
+
+
+# Ensure that a standby is able to follow a master on a newer timeline
+# when WAL archiving is enabled.
+
+# Initialize master node
+my $node_master_2 = get_new_node('master_2');
+$node_master_2->init(allows_streaming => 1, has_archiving => 1);
+$node_master_2->append_conf(
+	'postgresql.conf', qq(
+wal_keep_size = 512MB
+));
+$node_master_2->start;
+
+# Take backup
+$node_master_2->backup($backup_name);
+
+# Create standby node
+my $node_standby_3 = get_new_node('standby_3');
+$node_standby_3->init_from_backup($node_master_2, $backup_name,
+	has_streaming => 1);
+
+# Restart master node in standby mode and promote it, switching it
+# to a new timeline.
+$node_master_2->set_standby_mode;
+$node_master_2->restart;
+$node_master_2->promote;
+
+# Start standby node, create some content on master and check its presence
+# in standby, to ensure that the timeline switch has been done.
+$node_standby_3->start;
+$node_master_2->safe_psql('postgres',
+	"CREATE TABLE tab_int AS SELECT 1 AS a");
+$node_master_2->wait_for_catchup($node_standby_3, 'replay',
+	$node_master_2->lsn('write'));
+
+my $result_2 =
+  $node_standby_3->safe_psql('postgres', "SELECT count(*) FROM tab_int");
+is($result_2, qq(1), 'check content of standby 3');
